/*
FILE: PS2Storage.cpp
DESCRIPTION: Implementation file for the PS2MemoryCard, PS2HardDrive, and PS2StorageManager classes
WRITTEN BY: Derek Nedelman (nedelman@netzero.net)
DATE: August 24, 2002
*/

//Includes---------------------------------------------------------------------
#include "PS2Storage.h"
#include <stdio.h>	//For sprintf()
#include <stdlib.h>	//For system()
#include <sys/vfs.h>	//For statfs() which gets the hard drive stats


//Implementation---------------------------------------------------------------

//PS2MemoryCard----------------------------------------------------------------
bool PS2MemoryCard::Open(const char* name, int readOrWrite)
{
	if (!file)
		file = memcardlib_open(index, name, readOrWrite);
		
	return file != 0;
}

void PS2MemoryCard::Close()
{
	if (file)
		memcardlib_close(file);
}

int PS2MemoryCard::Read(void* data, int numBytes)
{
	if (file)
		return memcardlib_read(file, data, numBytes);
	else
		return 0;
}

int PS2MemoryCard::Write(const void* data, int numBytes)
{
	if (file)
		return memcardlib_write(file, data, numBytes);
	else
		return 0;
}

int PS2MemoryCard::GetPos() const
{
	if (file)
		return memcardlib_get_pos(file);
	else
		return 0;
}

bool PS2MemoryCard::FileExists(const char* name) const
{
	return memcardlib_file_exists(index, name) == 1;
}

void PS2MemoryCard::DeleteFile(const char* name) const
{
	memcardlib_delete_file(index, name);
}

int PS2MemoryCard::GetType() const
{
	return memcardlib_get_card_type(index);
}

void PS2MemoryCard::GetBlockInfo(int* blockSize, int* totalBlocks, int* freeBlocks) const
{
	memcardlib_get_card_block_info(index, blockSize, totalBlocks, freeBlocks);
}

int PS2MemoryCard::GetFreeBytes() const
{
	int blockSize, freeBlocks;

	GetBlockInfo(&blockSize, 0, &freeBlocks);

	return blockSize * freeBlocks;
}

int PS2MemoryCard::GetTotalBytes() const
{
	int blockSize, totalBlocks;

	GetBlockInfo(&blockSize, &totalBlocks, 0);

	return blockSize * totalBlocks;
}

bool PS2MemoryCard::IsFormatted() const
{
	return memcardlib_is_card_formatted(index) == 1;
}

int PS2MemoryCard::GetStatus() const
{
	return memcardlib_get_card_status(index);
}


//PS2HardDrive-----------------------------------------------------------------
void PS2HardDrive::GetBlockInfo(int* blockSize, int* totalBlocks, int* freeBlocks) const
{
	struct statfs stats;

	if (statfs("/", &stats) < 0)
	{
		if (blockSize)
			*blockSize = 0;
		if (totalBlocks)
			*totalBlocks = 0;
		if (freeBlocks)
			*freeBlocks = 0;
	}
	else
	{
		if (blockSize)
			*blockSize = stats.f_bsize;
		if (totalBlocks)
			*totalBlocks = stats.f_blocks;
		if (freeBlocks)
			*freeBlocks = stats.f_bfree;
	}
}

bool PS2HardDrive::IsFormatted() const
{
	return true;
}

void* PS2HardDrive::Open(const char* name, int readOrWrite)
{
	FILE* file;
	const char* openStrings[2] = {"rb", "wb"};

	file = fopen(name, openStrings[readOrWrite]);	
	return file;
}

void PS2HardDrive::Close(void* file)
{
	if (file)
		fclose((FILE*)file);
}

int PS2HardDrive::Read(void* file, void* data, int numBytes)
{
	return fread(data, 1, numBytes, (FILE*)file);
}

int PS2HardDrive::Write(void* file, const void* data, int numBytes)
{
	return fwrite(data, 1, numBytes, (FILE*)file);
}

int PS2HardDrive::GetPos(void* file) const
{
	return ftell((FILE*)file);
}

bool PS2HardDrive::FileExists(const char* name) const
{
	FILE* file;
	
	file = fopen(name, "rb");
	if (file)
	{
		fclose(file);
		return true;
	}
	else
		return false;
}

void PS2HardDrive::DeleteFile(const char* name) const
{
	char command_str[256];

	sprintf("rm -f %s", name);
	system(command_str);
}


//PS2StorageManager---------------------------------------------------------------

//Static initialization
int PS2StorageManager::numMemoryCards = 0;
PS2MemoryCard PS2StorageManager::memoryCards[PS2StorageManager::MAX_MEMCARDS];
PS2HardDrive PS2StorageManager::hardDrive;
bool PS2StorageManager::isInitialized = false;

void PS2StorageManager::Initialize()
{
	if (!isInitialized) 
	{ 
		int i; 
 
		memcardlib_initialize();
		numMemoryCards = memcardlib_num_supported_memcards();
 
		for (i = 0; i < numMemoryCards; i++) 
			memoryCards[i].index = i; 
 
		isInitialized = true; 
	} 
}

void PS2StorageManager::Shutdown()
{
	if (isInitialized)
	{
		memcardlib_shutdown();
		isInitialized = false;
	}
}

bool PS2StorageManager::IsHardDrivePresent()
{
	return true;
}

PS2HardDrive* PS2StorageManager::GetHardDrive()
{
	return &hardDrive;
}
