/*
FILE: main.cpp
DESCRIPTION: Test file demonstrating the usage of the PS2Storage library
WRITTEN BY: Derek Nedelman (nedelman@netzero.net)
DATE: August 24, 2002
*/


//Includes---------------------------------------------------------------------
#include <stdio.h>
#include "PS2Storage.h"


//Functions--------------------------------------------------------------------
void PrintStats(void)
{
	int i;
	int blockSize, totalBlocks, freeBlocks;

	for (i = 0; i < PS2StorageManager::GetNumMemoryCards(); i++)
	{	
		PS2MemoryCard* memCard = PS2StorageManager::GetMemoryCard(i);
	
		printf("Info for memord card %d\n", i);

		switch (memCard->GetType())
		{
			case PS2MC_TYPE_EMPTY: puts("\tNo memory card"); break;
			case PS2MC_TYPE_PS1: puts("\tPS1 memory card"); break;
			case PS2MC_TYPE_PS2: puts("\tPS2 memory card"); break;
			case PS2MC_TYPE_POCKETSTATION: puts("\tPocketstation"); break;
		}
			
		memCard->GetBlockInfo(&blockSize, &totalBlocks, &freeBlocks);
		printf("\tBlock size: %d\n", blockSize);
		printf("\tTotal blocks: %d\n", totalBlocks);
		printf("\tFree blocks: %d\n", freeBlocks);
		printf("\tTotal free memory: %d\n", memCard->GetFreeBytes());
		printf("\tTotal memory: %d\n", memCard->GetTotalBytes());

		switch (memCard->IsFormatted())
		{
			case 0: puts("\tMemory card is unformatted"); break;
			case 1: puts("\tMemory card is formatted"); break;
		}
		
		switch (memCard->GetStatus())
		{
			case PS2MC_STAT_NOTCON: puts("\tMemory card is not connected"); break;
			case PS2MC_STAT_READY: puts("\tMemory card is ready"); break;
			case PS2MC_STAT_BUSY: puts("\tMemory card is busy"); break;
			case PS2MC_STAT_ERROR: puts("\tMemory card error"); break;
		}
	}

	if (PS2StorageManager::IsHardDrivePresent())
	{
		PS2HardDrive* hardDrive = PS2StorageManager::GetHardDrive();

		printf("There is a hard drive present\n");
		
		hardDrive->GetBlockInfo(&blockSize, &totalBlocks, &freeBlocks);
		printf("\tBlock size: %d\n", blockSize);
		printf("\tTotal blocks: %d\n", totalBlocks);
		printf("\tFree blocks: %d\n", freeBlocks);
		
		switch (hardDrive->IsFormatted())
		{
			case 0: puts("\tHard drive is unformatted"); break;
			case 1: puts("\tHard drive is formatted"); break;
		}
	}
}

void ReadTest(PS2MemoryCard* card, const char* filename)
{
	char buffer[256] = {0};

	card->Open(filename, 0);

	if (card->IsOpen())
	{
		int numRead;
	
		printf("Opened %s on memory card %d for reading\n", filename, card->GetIndex());

		numRead = card->Read(buffer, sizeof(buffer));
		printf("Number of bytes read: %d\n", numRead);
		printf("Contents of file: %s\n", buffer);

		card->Close();
	}
	else
		printf("Couldn't open memory card %d for reading", card->GetIndex());
}

void WriteTest(PS2MemoryCard* card, const char* filename)
{
	char* buffer = "Text from the the memcardlib sample";

	card->Open(filename, 1);

	if (card->IsOpen())
	{
		int numWritten;
	
		printf("Opened %s on memory card %d for writing\n", filename, card->GetIndex());

		numWritten = card->Write(buffer, strlen(buffer));
		printf("Number of bytes written: %d\n", numWritten);
		printf("Contents of file: %s\n", buffer);

		card->Close();
	}
	else
		printf("Couldn't open memory card %d for writing", card->GetIndex());
}

int main(int argc, char** argv)
{
	PS2MemoryCard* memCard;
	int runMode = 0;	//0 for read, 1 for write, 2 for delete, 3 for stats
	const char* filename = "PS2StorageTest.txt";	//Name of file to create

	//The user must pass in an argument for the program to run	
	if (argc < 2)
	{
		puts("Usage: main {read | write | delete | stats}");
		return 0;
	}

	//Parse arguments
	if (!strcmp(argv[1], "read"))
		runMode = 0;
	else if (!strcmp(argv[1], "write"))
		runMode = 1;
	else if (!strcmp(argv[1], "delete"))
		runMode = 2;
	else if (!strcmp(argv[1], "stats"))
		runMode = 3;
	else
	{
		puts("Usage: main {read | write | delete | stats}");
		return 0;
	}

	//Initialize storage library
	PS2StorageManager::Initialize();

	memCard = PS2StorageManager::GetMemoryCard(0);

	//If there's no memory card in the selected slot, bail out
	if (memCard->GetType() == PS2MC_TYPE_EMPTY)
	{
		printf("A memory card must be in slot %d for this sample to work", memCard->GetIndex());
		PS2StorageManager::Shutdown();
		return 0;
	}		
	
	//Run the desired operation
	switch (runMode)
	{
		case 0: 
		{
			if (memCard->FileExists(filename))
			{
				printf("%s exists on memory card %d\n", filename, memCard->GetIndex());
				ReadTest(memCard, filename); 
			}
			else
				printf("%s doesn't exist on memory card %d\n", filename, memCard->GetIndex());

			break;
		}
		case 1: WriteTest(memCard, filename); break;
		case 2: 
		{
			if (memCard->FileExists(filename))
			{
				memCard->DeleteFile(filename);
				printf("%s deleted from memory card %d\n", filename, memCard->GetIndex());
			}
			else
				printf("%s doesn't exist on memory card %d\n", filename, memCard->GetIndex());
			break;
		}
		case 3: PrintStats(); break;
	}

	PS2StorageManager::Shutdown();
}

